package com.xxfc.platform.tour.rest.admin;

import com.github.wxiaoqi.security.common.msg.ObjectRestResponse;
import com.github.wxiaoqi.security.common.rest.BaseController;
import com.github.wxiaoqi.security.common.vo.PageDataVO;
import com.xxfc.platform.tour.biz.TourTagBiz;
import com.xxfc.platform.tour.entity.TourTag;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiResponse;
import org.springframework.web.bind.annotation.*;


/**
 * @author libin
 * @version 1.0
 * @description 旅游标签
 * @data 2019/6/11 9:57
 */
@RestController
@RequestMapping("/admin/tourTag")
@Api(value = "后台*旅游标签")
public class TourTagAdminController extends BaseController<TourTagBiz, TourTag> {

    /**
     * 分页查询标签
     *
     * @param pageNo   访问页
     * @param pageSize 分页大小
     * @return
     */
    @ApiImplicitParams(value = {
            @ApiImplicitParam(name = "pageNo",paramType = "query",dataType = "integer",defaultValue = "0"),
            @ApiImplicitParam(name = "pageSize",paramType = "query",dataType = "integer",defaultValue = "10")
    })
    @ApiResponse(code = 200,message = "Success")
    @GetMapping("/pages")
    public ObjectRestResponse<PageDataVO> findTourTageWithPage(@RequestParam(name = "pageNoe", defaultValue = "0",required = false) Integer pageNo,
                                                               @RequestParam(name = "pageSize", defaultValue = "10",required = false) Integer pageSize) {
        PageDataVO<TourTag> pageDataVO = getBaseBiz().findTourTagWithPage(pageNo, pageSize);
        return ObjectRestResponse.succ(pageDataVO);
    }

    /**
     * 根据 id 查询旅游标签
     * @param id
     * @return
     */
    @Override
    public ObjectRestResponse<TourTag> get(@PathVariable int id) {
     TourTag tourTag = getBaseBiz().findTourTagById(id);
     return ObjectRestResponse.succ(tourTag);
    }

    /**
     * 保存旅游标签
     * @param tourTag
     * @return
     */
    @Override
    public ObjectRestResponse<TourTag> add(@RequestBody TourTag tourTag) {
        int insertRows = getBaseBiz().saveTourTag(tourTag);
        if (insertRows>0){
            return  ObjectRestResponse.succ();
        }
        return ObjectRestResponse.createDefaultFail();
    }

    /**
     * 逻辑删除
     * @param id
     * @return
     */
    @Override
    public ObjectRestResponse<TourTag> remove(@PathVariable int id) {
        int effRow = getBaseBiz().updateTourTagStateById(id,1);

        if (effRow>0){
            return ObjectRestResponse.succ();
        }
        return ObjectRestResponse.createDefaultFail();

    }
}